@@ -20,55 +20,56 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/suncg14/cg14_driver.c,v 1.6 2001/10/01 13:44:10 eich Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <unistd.h>
+#include <sys/ioctl.h>
+#include <string.h>
+
 #include "xf86.h"
 #include "xf86_OSproc.h"
-#include "xf86_ansic.h"
-#include "xf86Version.h"
 #include "mipointer.h"
-#include "mibstore.h"
 #include "micmap.h"
 
 #include "fb.h"
 #include "xf86cmap.h"
 #include "cg14.h"
 
+#include "compat-api.h"
+
 static const OptionInfoRec * CG14AvailableOptions(int chipid, int busid);
 static void	CG14Identify(int flags);
 static Bool	CG14Probe(DriverPtr drv, int flags);
 static Bool	CG14PreInit(ScrnInfoPtr pScrn, int flags);
-static Bool	CG14ScreenInit(int Index, ScreenPtr pScreen, int argc,
-			      char **argv);
-static Bool	CG14EnterVT(int scrnIndex, int flags);
-static void	CG14LeaveVT(int scrnIndex, int flags);
-static Bool	CG14CloseScreen(int scrnIndex, ScreenPtr pScreen);
+static Bool	CG14ScreenInit(SCREEN_INIT_ARGS_DECL);
+static Bool	CG14EnterVT(VT_FUNC_ARGS_DECL);
+static void	CG14LeaveVT(VT_FUNC_ARGS_DECL);
+static Bool	CG14CloseScreen(CLOSE_SCREEN_ARGS_DECL);
 static Bool	CG14SaveScreen(ScreenPtr pScreen, int mode);
 static void	CG14InitCplane24(ScrnInfoPtr pScrn);
 static void	CG14ExitCplane24(ScrnInfoPtr pScrn);
 
 /* Required if the driver supports mode switching */
-static Bool	CG14SwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
+static Bool	CG14SwitchMode(SWITCH_MODE_ARGS_DECL);
 /* Required if the driver supports moving the viewport */
-static void	CG14AdjustFrame(int scrnIndex, int x, int y, int flags);
+static void	CG14AdjustFrame(ADJUST_FRAME_ARGS_DECL);
 
 /* Optional functions */
-static void	CG14FreeScreen(int scrnIndex, int flags);
-static ModeStatus CG14ValidMode(int scrnIndex, DisplayModePtr mode,
+static void	CG14FreeScreen(FREE_SCREEN_ARGS_DECL);
+static ModeStatus CG14ValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode,
 				Bool verbose, int flags);
 
 void CG14Sync(ScrnInfoPtr pScrn);
 
-#define VERSION 4000
+#define CG14_VERSION 4000
 #define CG14_NAME "SUNCG14"
 #define CG14_DRIVER_NAME "suncg14"
-#define CG14_MAJOR_VERSION 1
-#define CG14_MINOR_VERSION 0
-#define CG14_PATCHLEVEL 0
+#define CG14_MAJOR_VERSION PACKAGE_VERSION_MAJOR
+#define CG14_MINOR_VERSION PACKAGE_VERSION_MINOR
+#define CG14_PATCHLEVEL PACKAGE_VERSION_PATCHLEVEL
 
 /* 
  * This contains the functions needed by the server after loading the driver
@@ -79,7 +80,7 @@ void CG14Sync(ScrnInfoPtr pScrn);
  */
 
 _X_EXPORT DriverRec SUNCG14 = {
-    VERSION,
+    CG14_VERSION,
     CG14_DRIVER_NAME,
     CG14Identify,
     CG14Probe,
@@ -154,21 +155,17 @@ CG14GetRec(ScrnInfoPtr pScrn)
     if (pScrn->driverPrivate != NULL)
 	return TRUE;
 
-    pScrn->driverPrivate = xnfcalloc(sizeof(Cg14Rec), 1);
+    pScrn->driverPrivate = XNFcallocarray(sizeof(Cg14Rec), 1);
     return TRUE;
 }
 
 static void
 CG14FreeRec(ScrnInfoPtr pScrn)
 {
-    Cg14Ptr pCg14;
-
     if (pScrn->driverPrivate == NULL)
 	return;
 
-    pCg14 = GET_CG14_FROM_SCRN(pScrn);
-
-    xfree(pScrn->driverPrivate);
+    free(pScrn->driverPrivate);
     pScrn->driverPrivate = NULL;
 
     return;
@@ -238,7 +235,7 @@ CG14Probe(DriverPtr drv, int flags)
 		   devSections, numDevSections,
 		   drv, &usedChips);
 				    
-    xfree(devSections);
+    free(devSections);
     if (numUsed <= 0)
 	return FALSE;
 
@@ -257,7 +254,7 @@ CG14Probe(DriverPtr drv, int flags)
 	    pScrn = xf86AllocateScreen(drv, 0);
 
 	    /* Fill in what we can of the ScrnInfoRec */
-	    pScrn->driverVersion = VERSION;
+	    pScrn->driverVersion = CG14_VERSION;
 	    pScrn->driverName	 = CG14_DRIVER_NAME;
 	    pScrn->name		 = CG14_NAME;
 	    pScrn->Probe	 = CG14Probe;
@@ -272,9 +269,9 @@ CG14Probe(DriverPtr drv, int flags)
 	    xf86AddEntityToScreen(pScrn, pEnt->index);
 	    foundScreen = TRUE;
 	}
-	xfree(pEnt);
+	free(pEnt);
     }
-    xfree(usedChips);
+    free(usedChips);
     return foundScreen;
 }
 
@@ -331,12 +328,13 @@ CG14PreInit(ScrnInfoPtr pScrn, int flags)
     deal with depth
     *********************/
     
-    if (!xf86SetDepthBpp(pScrn, 32, 0, 32, Support32bppFb)) {
-	return FALSE;
-    } else {
-	/* Check that the returned depth is one we support */
-	switch (pScrn->depth) {
+    if (!xf86SetDepthBpp(pScrn, 0, 0, 0, Support24bppFb|Support32bppFb))
+		return FALSE;
+    /* Check that the returned depth is one we support */
+    switch (pScrn->depth) {
 	case 32:
+	case 24:
+	case 8:
 	    /* OK */
 	    break;
 	default:
@@ -344,13 +342,12 @@ CG14PreInit(ScrnInfoPtr pScrn, int flags)
 		       "Given depth (%d) is not supported by this driver\n",
 		       pScrn->depth);
 	    return FALSE;
-	}
     }
 
     /* Collect all of the relevant option flags (fill in pScrn->options) */
     xf86CollectOptions(pScrn, NULL);
     /* Process the options */
-    if (!(pCg14->Options = xalloc(sizeof(CG14Options))))
+    if (!(pCg14->Options = malloc(sizeof(CG14Options))))
 	return FALSE;
     memcpy(pCg14->Options, CG14Options, sizeof(CG14Options));
     xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, pCg14->Options);
@@ -360,7 +357,7 @@ CG14PreInit(ScrnInfoPtr pScrn, int flags)
      * xf86SetWeight references it.
      */
     if (pScrn->depth > 8) {
-	rgb weight = {10, 11, 11};
+	rgb weight = {0, 0, 0};
 	rgb mask = {0xff, 0xff00, 0xff0000};
                                        
 	if (!xf86SetWeight(pScrn, weight, mask)) {
@@ -425,7 +422,7 @@ CG14PreInit(ScrnInfoPtr pScrn, int flags)
 /* This gets called at the start of each server generation */
 
 static Bool
-CG14ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+CG14ScreenInit(SCREEN_INIT_ARGS_DECL)
 {
     ScrnInfoPtr pScrn;
     Cg14Ptr pCg14;
@@ -435,7 +432,7 @@ CG14ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     /* 
      * First get the ScrnInfoRec
      */
-    pScrn = xf86Screens[pScreen->myNum];
+    pScrn = xf86ScreenToScrn(pScreen);
 
     pCg14 = GET_CG14_FROM_SCRN(pScrn);
 
@@ -466,9 +463,11 @@ CG14ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
      */
     miClearVisualTypes();
 
-    /* Setup the visuals we support. */
+    /* Set the bits per RGB for 8bpp mode */
+    pScrn->rgbBits = 8;
 
-    if (!miSetVisualTypes(pScrn->depth, TrueColorMask,
+    /* Setup the visuals we support. */
+    if (!miSetVisualTypes(pScrn->depth, miGetDefaultVisualMask(pScrn->depth),
 			  pScrn->rgbBits, pScrn->defaultVisual))
 	return FALSE;
 
@@ -487,7 +486,6 @@ CG14ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     if (!ret)
 	return FALSE;
 
-    miInitializeBackingStore(pScreen);
     xf86SetBackingStore(pScreen);
     xf86SetSilkenMouse(pScreen);
 
@@ -520,6 +518,9 @@ CG14ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     if (!miCreateDefColormap(pScreen))
 	return FALSE;
 
+    if(!xf86SbusHandleColormaps(pScreen, pCg14->psdp))
+	return FALSE;
+
     pCg14->CloseScreen = pScreen->CloseScreen;
     pScreen->CloseScreen = CG14CloseScreen;
     pScreen->SaveScreen = CG14SaveScreen;
@@ -539,7 +540,7 @@ CG14ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
 /* Usually mandatory */
 static Bool
-CG14SwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
+CG14SwitchMode(SWITCH_MODE_ARGS_DECL)
 {
     return TRUE;
 }
@@ -551,7 +552,7 @@ CG14SwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
  */
 /* Usually mandatory */
 static void 
-CG14AdjustFrame(int scrnIndex, int x, int y, int flags)
+CG14AdjustFrame(ADJUST_FRAME_ARGS_DECL)
 {
     /* we don't support virtual desktops */
     return;
@@ -564,9 +565,9 @@ CG14AdjustFrame(int scrnIndex, int x, int y, int flags)
 
 /* Mandatory */
 static Bool
-CG14EnterVT(int scrnIndex, int flags)
+CG14EnterVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
 
     CG14InitCplane24 (pScrn);
     return TRUE;
@@ -579,9 +580,9 @@ CG14EnterVT(int scrnIndex, int flags)
 
 /* Mandatory */
 static void
-CG14LeaveVT(int scrnIndex, int flags)
+CG14LeaveVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
 
     CG14ExitCplane24 (pScrn);
     return;
@@ -595,21 +596,26 @@ CG14LeaveVT(int scrnIndex, int flags)
 
 /* Mandatory */
 static Bool
-CG14CloseScreen(int scrnIndex, ScreenPtr pScreen)
+CG14CloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     Cg14Ptr pCg14 = GET_CG14_FROM_SCRN(pScrn);
+    int bpp;
 
+    if (pScrn->bitsPerPixel > 8) {
+	bpp = 32;
+    } else
+	bpp = 8;
     pScrn->vtSema = FALSE;
+    CG14ExitCplane24 (pScrn);
     xf86UnmapSbusMem(pCg14->psdp, pCg14->fb,
-		     (pCg14->psdp->width * pCg14->psdp->height * 4));
+		     (pCg14->psdp->width * pCg14->psdp->height * (bpp >> 3)));
     xf86UnmapSbusMem(pCg14->psdp, pCg14->x32,
 		     (pCg14->psdp->width * pCg14->psdp->height));
     xf86UnmapSbusMem(pCg14->psdp, pCg14->xlut, 4096);
     
     pScreen->CloseScreen = pCg14->CloseScreen;
-    return (*pScreen->CloseScreen)(scrnIndex, pScreen);
-    return FALSE;
+    return (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 }
 
 
@@ -617,9 +623,10 @@ CG14CloseScreen(int scrnIndex, ScreenPtr pScreen)
 
 /* Optional */
 static void
-CG14FreeScreen(int scrnIndex, int flags)
+CG14FreeScreen(FREE_SCREEN_ARGS_DECL)
 {
-    CG14FreeRec(xf86Screens[scrnIndex]);
+    SCRN_INFO_PTR(arg);
+    CG14FreeRec(pScrn);
 }
 
 
@@ -627,7 +634,7 @@ CG14FreeScreen(int scrnIndex, int flags)
 
 /* Optional */
 static ModeStatus
-CG14ValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
+CG14ValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose, int flags)
 {
     if (mode->Flags & V_INTERLACE)
 	return(MODE_BAD);
@@ -640,10 +647,26 @@ CG14ValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 /* Mandatory */
 static Bool
 CG14SaveScreen(ScreenPtr pScreen, int mode)
-    /* this function should blank the screen when unblank is FALSE and
-       unblank it when unblank is TRUE -- it doesn't actually seem to be
-       used for much though */
 {
+#ifdef FBIOSVIDEO
+    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    Cg14Ptr pCg14 = GET_CG14_FROM_SCRN(pScrn);
+    int state;
+    switch(mode) {
+	case SCREEN_SAVER_ON:
+	case SCREEN_SAVER_CYCLE:
+		state = FBVIDEO_OFF;
+		ioctl(pCg14->psdp->fd, FBIOSVIDEO, &state);
+		break;
+	case SCREEN_SAVER_OFF:
+	case SCREEN_SAVER_FORCER:
+		state = FBVIDEO_ON;
+		ioctl(pCg14->psdp->fd, FBIOSVIDEO, &state);
+		break;
+	default:
+		return FALSE;
+    }
+#endif
     return TRUE;
 }
 
@@ -666,9 +689,12 @@ CG14InitCplane24(ScrnInfoPtr pScrn)
   int size, bpp;
               
   size = pScrn->virtualX * pScrn->virtualY;
-  bpp = 32;
+  if (pScrn->bitsPerPixel > 8) {
+  	bpp = 32;
+  } else
+  	bpp = 8;
   ioctl (pCg14->psdp->fd, CG14_SET_PIXELMODE, &bpp);
-  memset (pCg14->fb, 0, size * 4);
+  memset (pCg14->fb, 0, size * (bpp >> 3));
   memset (pCg14->x32, 0, size);
   memset (pCg14->xlut, 0, 0x200);
 }                                                  
@@ -683,4 +709,4 @@ CG14ExitCplane24(ScrnInfoPtr pScrn)
   int bpp = 8;
               
   ioctl (pCg14->psdp->fd, CG14_SET_PIXELMODE, &bpp);
-}                                                  
+}
